@@ -55,7 +55,7 @@ class PreArgParseSetupTest(unittest.TestCase):
             memory_handler, 1, 2, 3, debug=True, log_path=mock.ANY)
 
 
-class PostArgParseSetupTest(test_util.TempDirTestCase):
+class PostArgParseSetupTest(test_util.ConfigTestCase):
     """Tests for certbot.log.post_arg_parse_setup."""
 
     @classmethod
@@ -65,9 +65,10 @@ class PostArgParseSetupTest(test_util.TempDirTestCase):
 
     def setUp(self):
         super(PostArgParseSetupTest, self).setUp()
-        self.config = mock.MagicMock(
-            debug=False, logs_dir=self.tempdir, max_log_backups=1000,
-            quiet=False, verbose_count=constants.CLI_DEFAULTS['verbose_count'])
+        self.config.debug = False
+        self.config.max_log_backups = 1000
+        self.config.quiet = False
+        self.config.verbose_count = constants.CLI_DEFAULTS['verbose_count']
         self.devnull = open(os.devnull, 'w')
 
         from certbot.log import ColoredStreamHandler
@@ -102,7 +103,7 @@ class PostArgParseSetupTest(test_util.TempDirTestCase):
         self.assertFalse(os.path.exists(self.temp_path))
         mock_sys.excepthook(1, 2, 3)
         mock_except_hook.assert_called_once_with(
-            1, 2, 3, debug=self.config.debug, log_path=self.tempdir)
+            1, 2, 3, debug=self.config.debug, log_path=self.config.logs_dir)
 
         level = self.stream_handler.level
         if self.config.quiet:
@@ -119,7 +120,7 @@ class PostArgParseSetupTest(test_util.TempDirTestCase):
         self.test_common()
 
 
-class SetupLogFileHandlerTest(test_util.TempDirTestCase):
+class SetupLogFileHandlerTest(test_util.ConfigTestCase):
     """Tests for certbot.log.setup_log_file_handler."""
 
     @classmethod
@@ -129,7 +130,7 @@ class SetupLogFileHandlerTest(test_util.TempDirTestCase):
 
     def setUp(self):
         super(SetupLogFileHandlerTest, self).setUp()
-        self.config = mock.MagicMock(logs_dir=self.tempdir, max_log_backups=42)
+        self.config.max_log_backups = 42
 
     @mock.patch('certbot.main.logging.handlers.RotatingFileHandler')
     def test_failure(self, mock_handler):
@@ -155,7 +156,7 @@ class SetupLogFileHandlerTest(test_util.TempDirTestCase):
         handler.close()
 
         self.assertEqual(handler.level, logging.DEBUG)
-        self.assertEqual(handler.formatter.converter, time.gmtime)
+        self.assertEqual(handler.formatter.converter, time.localtime)
 
         expected_path = os.path.join(self.config.logs_dir, log_file)
         self.assertEqual(log_path, expected_path)
@@ -342,11 +343,17 @@ class PostArgParseExceptHookTest(unittest.TestCase):
     def _test_common(self, error_type, debug):
         """Returns the mocked logger and stderr output."""
         mock_err = six.StringIO()
+
+        def write_err(*args, **unused_kwargs):
+            """Write error to mock_err."""
+            mock_err.write(args[0])
+
         try:
             raise error_type(self.error_msg)
         except BaseException:
             exc_info = sys.exc_info()
             with mock.patch('certbot.log.logger') as mock_logger:
+                mock_logger.error.side_effect = write_err
                 with mock.patch('certbot.log.sys.stderr', mock_err):
                     try:
                         # pylint: disable=star-args
